#!/usr/bin/env python3
'''
Draw plot waveform
'''
import argparse
import math
import mimetypes
import os
import shutil
import sys
import threading
import glob

import magic

if __name__ != '__main__':
    sys.exit(0)

def drawBurst(i, Nbursts, idx_burst, length, x0, x1, y0, y1):
    '''
    Plot waveform
    '''
    cmd = "draw2Dcurve.py restored.asc_{}.asc --xName= --yName= --title='Burst {}/{} (length: {} samples)' --style=r,--,b,- --markers=0 --legend=Restored,Original --x0={} --x1={} --y0={} --y1={} --output=restored.asc_{}.png".format(i, idx_burst, Nbursts, length, x0, x1, y0, y1, i)
    os.system(cmd)
	
	
parser = argparse.ArgumentParser()
parser.add_argument("k", type=float)
parser.add_argument("b", type=int)
parser.add_argument("p", type=int)
parser.add_argument("nwindow", type=int)
parser.add_argument("action", type=int)
args = parser.parse_args()

#read program arguments

k = args.k
b = args.b
p = args.p
nwindow = args.nwindow
action = args.action

mime = magic.Magic(mime=True)

input_mime = mime.from_file("input_0.wav")
mtype, _ = input_mime.split('/')

# Check that the input is audio
if mtype != 'audio':
    with open('demo_failure.txt', 'w') as f:
        f.write('You must upload an audio file, not {}'.format(input_mime))
    sys.exit(0)

input_ext = mimetypes.guess_extension(input_mime)

if input_mime != 'audio/x-wav':
    os.rename("input_0.wav", "input_0{}".format(input_ext))
    os.system(
        "ffmpeg -y -i input_0{} input_0.wav 2> avconv_out.txt".format(input_ext))

#choose parameters depending on user action
if action == 0: # run default
    if nwindow < 8*p // 3:
        nwindow = 8*p // 3
    nwindow = int(4*math.ceil(nwindow/4))
elif action == 1: # default Nw
    nwindow = 8*p


print([f for f in glob.glob("*")])	
	
#execute preprocess
cmd = "ex_preprocess input_0.wav outSound=input_1.wav outASCII=input_1_big.asc > preprocess_stdout.txt"
os.system(cmd)
#returned_value = subprocess.call(cmd, shell=True)

#execute main algorithm
cmd = "ex_AR2 input_1.wav K={} b={} p={} nWindow={} nBurstsPlot=20 outSound=restored.wav outASCIIRestored=restored.asc outASCIIBursts=bursts.asc > ar2_stdout.txt".format(k, b, p, nwindow)
os.system(cmd)
#returned_value = subprocess.call(cmd, shell=True)

#extract number of bursts
with open("ar2_stdout.txt") as f:
    last_line = f.readlines()[-1]
    line = last_line.split()
    nburstsg = int(line[0])
   
# line = line.split()
# nbursts1 = int(line[6]);
# nsamples1 = float(line[7]);
# min1 = int(line[8]);
# max1 = int(line[9]);
# mean1 = float(line[10]);
# nbursts2 = int(line[11]);
# nsamples2 = float(line[12]);
# min2 = int(line[13]);
# max2 = int(line[14]);
# mean2 = float(line[15]);
# nburstsg = int(line[16]);
# nsamplesg = float(line[17]);
# ming = int(line[18]);
# maxg = int(line[19]);
# meang = float(line[20]);
# nburstplot = int(line[21]);


#read bursts metadata
with open("bursts.asc") as f:
    lines = f.readlines()
    nburstplot = len(lines)

#draw each burst
for i, line in enumerate(lines):
    x0, x1, y0, y1, idx_burst, length = line.split()
    t = threading.Thread(target=drawBurst, args=(i, nburstsg, idx_burst, length, x0, x1, y0, y1,))
    t.start()

t_main = threading.main_thread()
for t in threading.enumerate():
    if t != t_main:
        t.join()

#send data to the interface
key = os.path.basename(os.getcwd())
with open("algo_info.txt", "w") as f:
    f.write("nburstplot={}\n".format(nburstplot))
    f.write("work_dir={}\n".format("/api/core/shared_folder/run/5555000123/" + key + "/"))

# ------------------------------------------------------------------------------------------------
#subsample input signal for drawing
# with open("input_1_big.asc") as f_in:
#     lines_in = f_in.readlines()

# i = 0
# num_samples = len(lines_in)
# skip = num_samples / 640 if num_samples > 2000 else 1
# skip = int(skip)
# with open("input_1.asc", "w") as f_out:
#     for line in lines_in:
#         if i % skip == 0:
#             f_out.write(line)
#         i += 1

# with open("preprocess_stdout.txt") as f:
#     bounds_str = f.read()
# bounds_split = bounds_str.split(' ')
# bounds = int(bounds_split[0]), int(bounds_split[1])
#
# cmd = "draw2Dcurve.py input_1.asc --xName= --yName= --title=waveform --style=b,- --markers=0 --x0={} --x1={} --output=input_1.png".format(bounds[0], bounds[1])
# os.system(cmd)
